#include <Wire.h>
#include <Adafruit_PWMServoDriver.h>

Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();

#define SERVOMIN  125 // this is the 'minimum' pulse length count (out of 4096)
#define SERVOMAX  575 // this is the 'maximum' pulse length count (out of 4096)

const int PAUSE = 1000;
const int PAUSE_KL = 250;

class Segment{

 private:
  String name;
  int port;
  int maxAngle;
  int minAngle;

 public:
   Segment(String n, int p,int maxA, int minA){
     name = n;
     port = p;
     maxAngle = maxA;
     minAngle = minA;
   }

   String getName(){ return name; }
   int getPort(){ return port; }
   int getMaxAngle(){ return maxAngle; }
   int getMinAngle(){ return minAngle; }
};

Segment segmentA = Segment("Segment A", 0, 180, 90);
Segment segmentB = Segment("Segment B", 1, 180, 100);
Segment segmentC = Segment("Segment C", 2, 160, 60);
Segment segmentD = Segment("Segment D", 3, 160, 60);
Segment segmentE = Segment("Segment E", 4, 170, 60);
Segment segmentF = Segment("Segment F", 5, 140, 60);
Segment segmentG = Segment("Segment G", 6, 160, 90);


void setup() {
  Serial.begin(9600);
  Serial.println("16 channel Servo test!");

  pwm.begin();  
  pwm.setPWMFreq(60);  // Analog servos run at ~60 Hz updates
}

void loop() {
  displayNum0();
  delay(PAUSE);
  displayNum1();
  delay(PAUSE);
}

void displayNum0(){
  reset();
  moveUp(segmentA);
  delay(PAUSE_KL);
  moveUp(segmentB);
  delay(PAUSE_KL);
  moveUp(segmentC);
  delay(PAUSE_KL);
  moveUp(segmentD);
  delay(PAUSE_KL);
  moveUp(segmentE);
  delay(PAUSE_KL);
  moveUp(segmentF);
}

void displayNum1(){
  reset();
  moveUp(segmentB);
  delay(PAUSE_KL);
  moveUp(segmentC);
}

void displayNum2(){
  reset();
  moveUp(segmentA);
  delay(PAUSE_KL);
  moveUp(segmentB);
  delay(PAUSE_KL);
  moveUp(segmentG);
  delay(PAUSE_KL);
  moveUp(segmentE);
  delay(PAUSE_KL);
  moveUp(segmentD);
  delay(PAUSE_KL);
}

void displayNum3(){
  reset();
  moveUp(segmentA);
  moveUp(segmentB);
  moveUp(segmentG);
  moveUp(segmentC);
  moveUp(segmentD);
}

void displayNum4(){
  reset();
  moveUp(segmentF);
  moveUp(segmentG);
  moveUp(segmentB);
  moveUp(segmentC);
}

void displayNum5(){
  reset();
  moveUp(segmentA);
  moveUp(segmentF);
  moveUp(segmentG);
  moveUp(segmentC);
  moveUp(segmentD);
}

void displayNum6(){
  reset();
  moveUp(segmentA);
  moveUp(segmentF);
  moveUp(segmentG);
  moveUp(segmentE);
  moveUp(segmentD);
  moveUp(segmentC);
}

void displayNum7(){
  reset();
  moveUp(segmentA);
  moveUp(segmentB);
  moveUp(segmentC);
}

void displayNum8(){
  reset();
  moveUp(segmentA);
  moveUp(segmentB);
  moveUp(segmentC);
  moveUp(segmentD);
  moveUp(segmentE);
  moveUp(segmentF);
  moveUp(segmentG);
}

void displayNum9(){
  reset();
  moveUp(segmentA);
  moveUp(segmentB);
  moveUp(segmentC);
  moveUp(segmentG);
  moveUp(segmentF);
}

void reset(){
  resetSegment(segmentA);
  delay(PAUSE_KL);
  resetSegment(segmentB);
  delay(PAUSE_KL);
  resetSegment(segmentC);
  delay(PAUSE_KL);
  resetSegment(segmentD);
  delay(PAUSE_KL);
  resetSegment(segmentE);
  delay(PAUSE_KL);
  resetSegment(segmentF);
  delay(PAUSE_KL);
  resetSegment(segmentG);
}

void resetSegment(Segment segment){
  moveServo("reset ->",segment.getName(),segment.getPort(), segment.getMaxAngle());
}

void moveUp(Segment segment){
  moveServo("moveUp ->",segment.getName(),segment.getPort(), segment.getMinAngle());
}

void moveDown(Segment segment){
  moveServo("moveDown ->",segment.getName(),segment.getPort(), segment.getMaxAngle());
}

void moveServo(String action, String name, int port, int angle){
  Serial.print(action);
  Serial.println(name);  
  pwm.setPWM(port, 0, angleToPulse(angle));
}

int angleToPulse(int ang){
   return map(ang,0, 180, SERVOMIN,SERVOMAX);
}
